#!/usr/bin/env python
# -*- coding:utf-8 -*-
import re

from ..exceptions import raise_exception

__empty = "\\s|\\u00a0|\\u0020|\\u3000"

__spec_all_re = re.compile(f"({__empty})*", re.U)
__spec_start_end_re = re.compile(f"^({__empty})*|({__empty})*$", re.U)

__slot__ = (__empty, __spec_all_re, __spec_start_end_re)


def trip_all(value: str) -> str:
    if isinstance(value, str):
        return __spec_all_re.sub("", value)
    raise_exception(TypeError, f"expect is 'str', got a {type(value).__name__}")


def trip(value: str or bytes) -> str:
    if isinstance(value, str):
        return __spec_start_end_re.sub("", value)
    raise_exception(TypeError, f"expect is 'str', got a {type(value).__name__}")


def is_empty(value: str) -> bool:
    """
    Judge whether the string is empty
    The first and last spaces will be removed before judgment
    """
    if isinstance(value, str):
        return trip_all(value) == ""
    return False


def is_not_empty(value: str) -> bool:
    """
    Judge whether the string is not empty
    The first and last spaces will be removed before judgment
    """
    return not is_empty(value)


def is_black(value: str) -> bool:
    """
    string is black,don't remove start and end spec
    """
    if isinstance(value, str):
        return value == ""
    return False


def is_not_black(value: str) -> bool:
    """
    string isn't black,don't remove start and end spec
    """
    return not is_black(value)


def contains(src: str, target: str) -> bool:
    """
    src contains target
    """
    if isinstance(src, str) and isinstance(target, str):
        return target in src
    return False


def not_contains(src: str, target: str) -> bool:
    """
    src not contains target
    """
    return not contains(src, target)


def trip_contains(src: str, target: str) -> bool:
    """
    after removing the leading and trailing spaces, determine that src contains target
    """
    if isinstance(src, str) and isinstance(target, str):
        return target.strip() in src.strip()
    return False


def trip_not_contains(src: str, target: str) -> bool:
    """
    after removing the leading and trailing spaces, determine that src not contains target
    """
    if isinstance(src, str) and isinstance(target, str):
        return target.strip() in src.strip()
    return not trip_contains(src, target)


def trip_all_contains(src: str, target: str) -> bool:
    if isinstance(src, str) and isinstance(target, str):
        return trip_all(target) in trip_all(src)
    return False


def trip_all_not_contains(src: str, target: str) -> bool:
    return not trip_all_contains(src, target)
