#!/usr/bin/env python
# -*- coding:utf-8 -*-
from random import randint
from typing import TypeVar, Iterable

T = TypeVar("T")


def none_of_default(src: T, default: T) -> T:
    """
    Judge whether SRC is empty, and return the value of default if it is empty
    :param src: Object to be judged
    :param default: Default value
    """
    if src:
        return src
    return default


def generate_random_str(length: int = 16) -> str:
    """
    Generates a random string of a specified length
    :params length: Length of generated string
    """
    random_str = ''
    base_str = 'ABCDEFGHIGKLMNOPQRSTUVWXYZabcdefghigklmnopqrstuvwxyz0123456789'
    base_str_length = len(base_str) - 1
    for i in range(length):
        random_str += base_str[randint(0, base_str_length)]
    return random_str


def check_none(value):
    """
    判断是否none，为none抛异常
    """
    if not value:
        raise AssertionError(f"expected not None, but got None")


def get_enum(enum_class: Iterable, value):
    """
    传入一个枚举类，并获取指定值的枚举对象
    :param enum_class: 枚举类
    :param value: 枚举对象的值
    """
    for k in enum_class:
        if k.value == value:
            return k
    return None
