#!/usr/bin/env python
# -*- coding:utf-8 -*-
from decimal import Decimal, ROUND_HALF_UP
from enum import Enum
from typing import Optional, Union

_T = Optional[Union[int, float, str, bool]]


class _Integer(int):
    def __new__(cls, num: _T = 0):
        return int.__new__(cls, num)

    def __init__(self, num: _T = 0):
        self.__num = num


class _Float(float):
    def __new__(cls, num: _T = 0):
        return float.__new__(cls, num)

    def __init__(self, num: _T = 0):
        self.__num = num


class _NumericType(_Float):
    def __init__(self, num: _T = 0):
        super().__init__(num)
        self.__num = num

    def round(self, accuracy: int = None) -> float:
        """
        Rounds floating-point types
        """
        if isinstance(accuracy, int) and accuracy >= 0:
            return float(Decimal(self.__num).quantize(Decimal(f'0.{"0" * accuracy}'), rounding=ROUND_HALF_UP))
        return self.__num

    def integer(self) -> int:
        """
        Output as int type
        """
        return int(self.__num)


class _Converter(_Integer):
    """
    The calculation is performed with bit as the reference unit
    """

    def __init__(self, num: int or float):
        self.__num: int = num
        self.__decimal = None
        super().__init__(num)

    def to(self, unit: Enum):
        return _NumericType(self.__num * self.__decimal / unit.value)
