#!/usr/bin/env python
# -*- coding:utf-8 -*-
from inspect import getfullargspec
from typing import TypeVar, Dict, List

T = TypeVar("T", bound=Dict)
R = TypeVar("R")


def around_impl(func: callable, before: Dict[callable, dict] or List[callable] or callable = None,
                after: Dict[callable, dict] or List[callable] or callable = None,
                catch: bool = False, deliver: bool = True, *args, **kwargs):
    func_origin_args = getfullargspec(func).args
    tmp_args = copy(args)
    for i in range(len(tmp_args)):
        kwargs[func_origin_args[i]] = tmp_args[i]
        args = args[:-1]
    all_func_result = _run_hook_func(before, R, deliver)
    func_params_map = {}
    __hook_params(func, func_params_map, all_func_result)
    kwargs.update(func_params_map)
    tmp = []
    # noinspection PyBroadException
    try:
        result = func(*args, **kwargs)
        tmp.append(result)
        return result
    except BaseException as e:
        if not catch:
            raise e
    finally:
        func_result = None
        if tmp:
            func_result = tmp[0]
        if deliver:
            _run_hook_func(after, func_result, deliver)
        else:
            all_func_result[func.__name__] = func_result
            _run_hook_func(after, all_func_result, deliver)


def _run_hook_func(call_obj: Dict[callable, dict] or List[callable] or callable,
                   func_return: R, deliver: bool) -> T:
    result_map = {}
    if not call_obj:
        return {}
    if isinstance(call_obj, dict):
        for call, params in call_obj.items():
            if callable(call):
                params_map = {}
                __hook_params(call, params_map, func_return)
                params.update(params_map)
                func_return = call(**params)
                result_map[call.__name__] = func_return
    elif isinstance(call_obj, list):
        for call in call_obj:
            if callable(call):
                result_map[call.__name__] = call()
    elif callable(call_obj):
        result_map[call_obj.__name__] = call_obj()
    if deliver:
        keys = list(result_map.keys())
        if keys:
            return result_map[keys[len(keys) - 1]]
        else:
            return R
    return result_map


def __hook_params(call: callable, params_map: dict, result: R):
    hook_func_args = getfullargspec(call).args
    hook_func_kwonlyargs = getfullargspec(call).kwonlyargs
    hook_func_all_args = hook_func_args + hook_func_kwonlyargs
    for arg in hook_func_all_args:
        params_map[arg] = None
    hook_func_kwonlydefaults = getfullargspec(call).kwonlydefaults
    if hook_func_kwonlydefaults:
        params_map.update(hook_func_kwonlydefaults)
    if "func_return" in params_map:
        params_map["func_return"] = result


def copy(values: tuple) -> tuple:
    tmp = []
    tmp_append = tmp.append
    for v in values:
        tmp_append(v)
    return tuple(tmp)
