#!/usr/bin/env python
# -*- coding:utf-8 -*-
from datetime import datetime
from pathlib import Path

from ..converter.storage import StorageUnit
from ..log.level import LogLevel


class __LogConfig(object):
    def __init__(self):
        self.__dir: Path = Path.home().joinpath("logs").absolute()
        self.__level: LogLevel = LogLevel.DEBUG
        self.__format: str = "[%(asctime)s]-[%(process)s]-[%(thread)s]-[%(filename)s %(lineno)s]-[%(levelname)s]: %(message)s"
        self.__code: str = "utf-8"
        self.__name: str = datetime.now().strftime('%Y-%m-%d') + ".log"
        self.__max_bytes: int = StorageUnit.MB.of(100).to(StorageUnit.BYTE).integer()
        self.__backup_count: int = 5
        self.__path: str = str(self.__dir.joinpath(self.__name).absolute())
        self.__banner: bool = True
        if not self.__dir.exists():
            self.dir.mkdir(parents=True)

    @property
    def dir(self) -> Path:
        return self.__dir

    @dir.setter
    def dir(self, value: str):
        path = Path(value)
        if not path.exists():
            path.mkdir(parents=True)
        self.__dir = value
        self.__path = Path(self.__dir).joinpath("logs", self.__name)

    @property
    def level(self) -> LogLevel:
        return self.__level

    @level.setter
    def level(self, value: LogLevel):
        self.__level = value

    @property
    def format(self) -> str:
        return self.__format

    @format.setter
    def format(self, value: str):
        self.__format = value

    @property
    def code(self) -> str:
        return self.__code

    @code.setter
    def code(self, value: str):
        self.__code = value

    @property
    def name(self) -> str:
        return self.__name

    @name.setter
    def name(self, value: str):
        self.__name = value
        self.__path = Path(self.dir).joinpath(self.__name)

    @property
    def max_bytes(self) -> int:
        return self.__max_bytes

    @max_bytes.setter
    def max_bytes(self, value: int):
        self.__max_bytes = value

    @property
    def backup_count(self) -> int:
        return self.__backup_count

    @backup_count.setter
    def backup_count(self, value: int):
        self.__backup_count = value

    @property
    def path(self) -> str:
        return self.__path

    @property
    def banner(self) -> bool:
        return self.__banner

    @banner.setter
    def banner(self, value: bool):
        self.__banner = value
